'use strict';
const { CrudService } = require('naf-framework-mongoose/lib/service');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const _ = require('lodash');
const jwt = require('jsonwebtoken');
const assert = require('assert');
// 机构
class OrganizationService extends CrudService {
  constructor(ctx) {
    super(ctx, 'organization');
    this.redis = this.app.redis;
    this.model = this.ctx.model.Organization;
  }
  /**
   * 创建用户
   * @param {Object} params 用户信息
   */
  async create({ password, ...data }) {
    data.password = { secret: password };
    const { institution_code } = data;
    // 检查是否重复
    const num = await this.model.count({ institution_code, isdel: '0' });
    if (num > 0) throw new BusinessError(ErrorCode.DATA_EXISTED, '已有个机构使用该 统一社会信用代码');
    return await this.model.create(data);
  }
  /**
   * 修改密码
   * @param {Object} {id,password} 用户id和密码
   */
  async password({ id, password }) {
    const object = await this.model.findById(id);
    if (!object) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '未找到用户的信息');
    object.password = { secret: password };
    await object.save();
  }
  /**
   * 登陆
   * @param {Object} params 登陆信息
   * @property institution_code 手机号
   * @property password 密码
   */
  async login({ institution_code, password }) {
    const object = await this.model.findOne({ institution_code, isdel: '0' }, '+password');
    if (!object) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '未找到用户的信息');
    const { password: op, status } = object;
    const { secret } = op;
    if (status !== '1') throw new BusinessError(ErrorCode.ACCESS_DENIED, '拒绝访问!');
    if (secret !== password) throw new BusinessError(ErrorCode.BAD_PASSWORD, '密码错误');
    const data = _.omit(JSON.parse(JSON.stringify(object)), [ 'meta', 'password', '__v' ]);
    const { secret: secrets } = this.config.jwt;
    const token = jwt.sign(data, secrets);
    // 记录登陆
    let number = await this.redis.get('login_number') || 0;
    number++;
    await this.redis.set('login_number', number);
    return token;
  }

  async delete({ id }) {
    const object = await this.model.findById(id);
    if (!object) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '未找到用户的信息');
    object.isdel = '1';
    await object.save();
  }
  /**
   * 用手机号获取企业列表
   * @param {Object} query phone:电话号
   * @param {Object} options skip;limit
   */
  async getList({ name, phone }, { skip = 0, limit = 0 } = {}) {
    assert(phone, '缺少手机号');
    const query = { phone, status: '1' };
    if (name) query.name = name;
    const list = await this.model.find(query).skip(parseInt(skip)).limit(parseInt(limit));
    return list;
  }

  /**
   * 企业绑定微信
   * @param {Object} body
   * @property id 企业id
   * @property openid 微信openid
   */
  async bind({ id, openid }) {
    await this.bindRemove({ openid });
    const org = await this.model.findById(id);
    if (!org) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '未找到指定企业');
    org.openid = openid;
    await org.save();
    return await org.save();
  }
  /**
   * 解除绑定
   * @param {Object} body
   * @property id 企业id
   * @property openid 微信id
   * 两种方式:id=>指定企业的openid解绑;openid=>删除所有企业的该openid
   */
  async bindRemove({ id, openid }) {
    if (id) {
      const org = await this.model.findById(id);
      if (!org) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '未找到指定企业');
      org.openid = undefined;
      return await org.save();
    }
    const res = await this.model.updateMany({ openid }, { openid: undefined });
    return res;
  }

  /**
   * 企业微信登录
   * @param {Object} body post body
   * @property openid
   */
  async wxLogin({ openid }) {
    const org = await this.model.findOne({ openid });
    if (!org) return org;
    const data = _.omit(JSON.parse(JSON.stringify(org)), [ 'meta', 'password', '__v' ]);
    const { secret: secrets } = this.config.jwt;
    const token = jwt.sign(data, secrets);
    return token;
  }
}

module.exports = OrganizationService;
